<?php // dirs.php

/** Singleton class responsible for finding relative paths.
 */
class Dirs
{
  var $checked;
  var $path_links;
  var $imagefiles;
  var $directories;
  var $top; ///< path to the top directory. E.g. ../..
  function Dirs()
  {
    $this->checked=array();
    $this->path_links=array(); // { name -> link }
    // Make sure that the _exa directory exists.
    $this->album('.');
    // Find out where we are.
    $path=getcwd();
    $link=array();
    while(TRUE)
    {
      if(!is_dir($path.'/_exa'))
          break;
      $L=implode('/',array_reverse($link));
      $this->top=strlen($L)?$L:'.';
      $this->path_links[hsc(basename($path))]=$this->top;
      $link[]='..';
      $newpath=dirname($path);
      if($newpath==$path)
          break;
      $path=$newpath;
    }
    $this->path_links=array_reverse($this->path_links,true);
    // Read the contents of the current directory.
    list($this->directories,$this->imagefiles)=$this->read_contents();
  }
  /** Returns the path of the current album dir. */
  function album($root)
  {
    $result=$root.'/_exa';
    if(!array_key_exists($result,$this->checked) && !is_dir($result))
    {
      mkdir($result,0777);
      chmod($result,0777);
      $this->checked[$result]=TRUE;
    }
    return $result;
  }
  /** Returns the relative path of the *top* _exa dir. */
  function top_album()
  {
    return $this->top.'/_exa';
  }
  /** Ensures that the named directory is exists, and return the path. */
  function get_dir($dirname)
  {
    if(!array_key_exists($dirname,$this->checked))
    {
      if(!is_dir($dirname))
      {
        mkdir($dirname,0777);
        chmod($dirname,0777);
      }
      $this->checked[$dirname]=TRUE;
    }
    return $dirname;
  }
  /** Returns the relative path to the _exa/<image>.d directory. */
  function image($image)
  {
    return $this->get_dir(
      $this->album($image->info['dirname']).'/'.$image->info['basename'].'.d'
    );
  }
  /** Returns the relative path FROM root TO here. */
  function path()
  {
    global $options;
    $result=array_keys($this->path_links);
    $result[0]=$options->top_name;
    return implode('/',$result);
  }
  /** Returns the relative path FROM root TO here, as a series of
   *  HTML links.
   *  Set parameter to
   *   - TRUE:    forces last link
   *   - FALSE:   suppresses last link
   *   - default: last is link if there are sub-directories
   */
  function path_links($link_last=1)
  {
    global $options;
    if($link_last!==TRUE && 0==count($this->directories))
      $link_last=FALSE;
    $result=array_keys($this->path_links);
    $result[0]=$options->top_name;
    $links=array_values($this->path_links);
    $len=count($result);
    for($i=0; $i<$len; $i++)
    {
      // Record where we came from, so that doc_dir() can go to the correct page
      if($i<$len-1)
        $from_dir='/index.php?from_dir='.urlencode($result[1+$i]);
      else
        $from_dir='';

      if($link_last || $i<$len-1)
        $result[$i]='<a href="'.urlencode($links[$i]).$from_dir.'">'
                   .$result[$i].'</a>';
      if($i==$len-1)
        $result[$i]='<strong>'.$result[$i].'</strong>';
    }
    $result=implode('/',$result);
    if(strlen($options->home_url))
    {
      $result=sprintf('<a href="%s">%s</a>&nbsp;&nbsp;'."\n",
                $options->home_url,
                $options->home_name
              ).$result;
    }
    return $result;
  }
  /** Returns a sorted list of images in $dirname. */
  function read_contents($dirname='.')
  {
    $imagefiles=array();
    $directories=array();
    if($handle=opendir($dirname))
    {
      while(FALSE !== ($fname=readdir($handle)))
      {
        if(is_dir($fname) && is_writeable($fname))
        {
          if(preg_match('/^[^._]/',$fname))
              $directories[]=$fname;
        }
        elseif($mime=image_mime($dirname.'/'.$fname))
        {
          $im=new ImageFile($dirname.'/'.$fname,$mime);
          //$imagefiles[$fname]=new ImageFile($dirname.'/'.$fname,$mime);
          $imagefiles[sprintf('%012d_%s',$im->date_time(),$fname)]=$im;
        }
      } // end while
      closedir($handle);
      ksort($imagefiles);
    }
    return array($directories,array_values($imagefiles));
  }
  function current_album_title()
  {
    global $options;
    if(count($this->path_links)>1)
      return nice(basename(getcwd()));
    else
      return $options->top_name;
  }
  
}
// Construct the global singleton.
$dirs=new Dirs();

?>
