<?php // imagefile.php

/** Representation of an image. */
class ImageFile
{
  var $fname;
  var $info;
  var $mime;
  var $name;
  var $exif='';
  var $exif_caption='';
  var $dt=-1; // date time (unixtime)
  function ImageFile($fname,$mime)
  {
    $this->fname=$fname;
    $this->info=pathinfo($fname);
    $this->mime=$mime;
    $b=$this->info['basename'];
    $e=$this->info['extension'];
    $this->name=substr($b,0,strlen($b)-strlen($e)-1);
    $this->read_exif();
  }
  
  /** Loads the image */
  function image()
  {
    if($this->mime=='image/jpeg')
        return imagecreatefromjpeg($this->fname);
    else if($this->mime=='image/png')
        return imagecreatefrompng($this->fname);
    else if($this->mime=='image/gif')
        return imagecreatefromgif($this->fname);
    else
        return FALSE;
  }
  
  /** Returns the square filename. */
  function square()
  {
    global $dirs;
    $result=$dirs->image($this).'/square.jpg';
    if(!file_exists($result) || filemtime($result)<filemtime($this->fname))
    {
      // Make a small square thumbnail.
      list($w0,$h0) = getimagesize($this->fname);
      if($w0<$h0)
      { // Tall
        $d0=$w0;
        $x0=0;
        $y0=($h0-$w0)/2;
      }
      else
      { // Wide
        $d0=$h0;
        $x0=($w0-$h0)/2;
        $y0=0;
      }
      // Zoom in a bit more.
      $fringe=$d0/10;
      $x0+=$fringe;
      $y0+=$fringe;
      $d0-=(2*$fringe);

      $image=$this->image();
      $square=imagecreatetruecolor(75,75);
      imagecopyresampled(
        $square,
        $image,
        0,0,   // dst x,y
        $x0,$y0, // src x,y
        75,75,
        $d0,$d0
      );
      // Save it.
      imagejpeg($square,$result,85);
      imagedestroy($square);
      chmod($result,0666);
    }
    return new ImageRef($result,75,75);
  }

  /** Returns the resized filename. */
  function resized($img_size)
  {
    global $dirs;
    
    if($img_size==0)
      return $this->square();
    elseif($img_size==1)
    {
      $name='small';
      $W=240;
      $H=240;
    }
    elseif($img_size==2)
    {
      $name='carousel';
      $W=500;
      $H=500;
    }
    elseif($img_size==3)
    {
      $name='medium';
      $W=800;
      $H=533;
    }
    elseif($img_size==4)
    {
      $name='large';
      $W=1024;
      $H=700;
    }
    else
    {
      return new ImageRef($this->fname);
    }
    
    $result=$dirs->image($this).'/'.$name.'.jpg';
    if(!file_exists($result) || filemtime($result)<filemtime($this->fname))
    {
      // Make a file.
      list($w0,$h0) = getimagesize($this->fname);
      if( $h0*$W/$w0 <= $H)
      {
        $w1=$W;
        $h1=$h0*$W/$w0;
      }
      else
      {
        $w1=$w0*$H/$h0;
        $h1=$H;
      }
      $image=$this->image();
      $resized=imagecreatetruecolor($w1,$h1);
      imagecopyresampled(
        $resized,
        $image,
        0,0, // dst x,y
        0,0, // src x,y
        $w1,$h1,
        $w0,$h0
      );
      // Save it.
      imagejpeg($resized,$result,85);
      imagedestroy($resized);
      chmod($result,0777);
    }
    return new ImageRef($result,$w1,$h1);
  }

  /** Reads the EXIF data for this image. */
  function read_exif()
  {
    if(function_exists('exif_read_data'))
    {
      // exif_read_data() can spew warnings, so suppress them with @
      $exif_data= @ exif_read_data($this->fname,'EXIF');
      if($exif_data===FALSE)
          return;
      
      // Create the summary string.
      $pieces=array();
      $fields=array('Model'=>'<strong>%s</strong>',
        'FocalLength'=>'%smm', // 'DateTime'=>'%s',
        'ExposureTime'=>'%ss', 'FNumber'=>'f%s', 'ISOSpeedRatings'=>'ISO%d');
      $rationals=array('FNumber','FocalLength');
      foreach($fields as $k => $templ)
      {
        if(array_key_exists($k,$exif_data))
        {
          $v=$exif_data[$k];
          if(in_array($k,$rationals))
            $v=rational2dec($v);
          $pieces[]=sprintf($templ,$v);
        }
      }
      $this->exif=implode(",\n",$pieces);

      // Read the date time
      if(array_key_exists('DateTime',$exif_data))
      {
        $date_time=explode(' ',$exif_data['DateTime']);
        if(count($date_time)>=2)
        {
          $d=explode(':',$date_time[0]);
          $t=explode(':',$date_time[1]);
          if(count($d)>=3 && count($t)>=3)
              $this->dt=mktime(
                 intval($t[0]),intval($t[1]),intval($t[2]),
                 intval($d[1]),intval($d[2]),intval($d[0])
              );
        }
      }
      
      // Read the EXIF comment
      if(array_key_exists('COMPUTED',$exif_data))
      {
        $c=$exif_data['COMPUTED'];
        if(isset($c['UserComment']) && isset($c['UserCommentEncoding']))
        {
          if('ASCII'==$c['UserCommentEncoding'])
              $this->exif_caption=hsc($c['UserComment']);
          // TODO: support Unicode & JIS encoding here.
        }
      }
    }
  }

  /** Returns the date and time (unix time) of the image, from Exif if
   *  possible, otherwise just the file timestamp. */
  function date_time()
  {
    if($this->dt==-1)
       $this->dt=filemtime($this->fname);
    return $this->dt;
  }

  /** Returns the caption for this image, or FALSE if there is none. */
  function caption()
  {
    global $dirs;
    $caption_fname=$dirs->image($this).'/caption.html';
    $result=false;
    if(file_exists($caption_fname))
    {
      $handle=fopen($caption_fname,'r');
      $caption=fread($handle,1024);
      if(strlen($caption)>0)
        $result=$caption;
      fclose($handle);
    }
    elseif(strlen($this->exif_caption))
    {
      $result=$this->exif_caption;
    }
    return $caption;
  }

  function short_caption($limit=40,$format='%s')
  {
    $caption=$this->caption();
    if(!$caption)
      return '';
    $result=explode("\n",$caption);
    $result=$result[0];
    if(strlen($result)>$limit)
    {
      $words=preg_split('/[\s]+/',$result);
      $c=0;
      for($i=0; $i<count($words) && $c<$limit; $i++)
        $c+=1+strlen($words[$i]);
      $result=implode(' ',array_slice($words,0,$i));
      if(strlen($result) < strlen($caption))
        $result.='...';
    }
    return sprintf($format,$result);
  }
  
  function setcaption($txt)
  {
    global $dirs;
    $caption_fname=$dirs->image($this).'/caption.html';
    $handle=fopen($caption_fname,'w');
    fwrite($handle,$txt);
    fclose($handle);
  }

  function numcomments($format='%s')
  {
    global $dirs;
    $result=0;
    $comments_dir=$dirs->image($this).'/comments';
    if(is_dir($comments_dir) && $dhandle=opendir($comments_dir))
    {
      while(FALSE !== ($fname=readdir($dhandle)))
        if($fname!='.' && $fname!='..')
          $result++;
      closedir($dhandle);
    }
    if($result==1)
      return sprintf($format,$result.'&nbsp;comment');
    elseif($result>1)
      return sprintf($format,$result.'&nbsp;comments');
    else
      return '';
  }

  function comments()
  {
    global $dirs;
    $result=array();
    $comments_dir=$dirs->image($this).'/comments';
    if(is_dir($comments_dir) && $dhandle=opendir($comments_dir))
    {
      while(FALSE !== ($fname=readdir($dhandle)))
      {
        $path=$comments_dir.'/'.$fname;
        if(is_file($path) && is_readable($path))
        {
          $result[$fname]=new Comment($path,$fname);
        }
      } // end while
      closedir($dhandle);
      if(count($result))
      {
        ksort($result);
        return $result;
      }
    }
    return FALSE;
  }

  function addcomment($author,$comment)
  {
    global $dirs;
    $comments_dir=$dirs->get_dir( $dirs->image($this).'/comments' );
    // If the same IP address submits two comments in the same second, then
    // we lose one. Ho hum.
    $fname=sprintf($comments_dir.'/%012d-%s',time(),$_SERVER['REMOTE_ADDR']);
    $handle=fopen($fname,"w");
    if($handle!==FALSE)
    {
      fwrite($handle,$author."\n".$comment);
      fclose($handle);
    }
  }

  function delcomment($cid)
  {
    global $dirs;
    $comments_dir=$dirs->get_dir( $dirs->image($this).'/comments' );
    $fname=$comments_dir.'/'.$cid;
    if(is_file($fname))
        unlink($fname);
  }

}

?>
