<?php // main.php

/** Dies if $arr does not contain $keys. */
function assert_keys($keys,$arr)
{
  foreach($keys as $k)
    if(!array_key_exists($k,$arr))
      die('Missing key: '.$k);
}


/** Utility, deals with input (from $_POST).
 *  Strips slashes and makes it HTML-safe. */
function safe_input($s)
{
  return htmlentities(stripslashes( $s ), ENT_QUOTES);
}


function process_form()
{
  global $is_admin,$options,$dirs,$img;
  $do=$_POST['do'];
  if('setpasswd'==$do)
  {
    assert_keys(array('p1','p2'),$_POST);
    $p1=$_POST['p1'];
    $p2=$_POST['p2'];
    if(strlen($options->passwd) && !$is_admin)
      admin_message('You must log-in to set the password.');
    elseif(strlen($p1) < 8)
      admin_message('Your password must be at least eight characters long.');
    elseif($p1!=$p2)
      admin_message('You must type the same password twice.');
    else
    {
      $options->passwd=md5($p1.$options->secret);
      $options->save();
      setcookie('exa_passwd',$options->passwd,0,'/');  
      $is_admin=true;
    }
  }
  elseif(intval($options->comments) && 'addcomment'==$do)
  {
    assert_keys(array('cmt_author','cmt_txt'),$_POST);
    $author=htmlentities(stripslashes( $_POST['cmt_author'] ));
    $comment=htmlentities(stripslashes( $_POST['cmt_txt'] ));
    if(strlen($author) && strlen($comment))
    {
      $dirs->imagefiles[$img]->addcomment($author,$comment);
      admin_message('Comment saved.');
    }
  }
  elseif(!$is_admin)
  {
    die('Not logged in.');
  }
  
  // MUST BE LOGGED-IN TO GET PAST THIS POINT

  if('setprefs'==$do)
  {
    $options->comments=intval($_POST['comments']);
    $options->exif=intval($_POST['exif']);
    $options->reload=intval($_POST['reload']);
    $options->top_name=safe_input($_POST['top_name']);
    $options->home_name=safe_input($_POST['home_name']);
    $options->home_url=safe_input($_POST['home_url']);
    $options->slideshow_delay=max(0,1000*intval($_POST['slideshow_delay']));
    $options->save();
    admin_message('Preferences saved.');
  }
  elseif('editcaption'==$do)
  {
    assert_keys(array('cap_txt'),$_POST);
    $caption=htmlentities(stripslashes( $_POST['cap_txt'] ));
    $dirs->imagefiles[$img]->setcaption($caption);
    admin_message('Caption saved.');
  }
  elseif('delcomment'==$do)
  {
    assert_keys(array('cid'),$_POST);
    $cid=safe_input($_POST['cid']);
    $dirs->imagefiles[$img]->delcomment($cid);
    admin_message('Comment deleted.');
  }
  elseif('delcomments'==$do)
  {
    assert_keys(array('cpaths'),$_POST);
    if(is_array($_POST['cpaths']))
        foreach($_POST['cpaths'] as $cpath)
            if(is_file($cpath) && 'comments'==basename(dirname($cpath)))
                unlink($cpath);
    admin_message('Comments deleted.');
  }
}

/** Force download of the current image
 *  (by setting MIME type to application/octet-stream). */
function download()
{
  assert_img();
  global $dirs,$img;
  $fname=$dirs->imagefiles[$img]->fname;
//  header('HTTP/1.1 200 OK');
  header('Date: ' . date("D M j G:i:s T Y"));
  header('Last-Modified: ' . date("D M j G:i:s T Y"));
  header("Content-Type: application/octet-stream");
  header("Content-Length: " . (string)(filesize($fname)) );
  header("Content-Transfer-Encoding: Binary");
  header('Content-Disposition: attachment; filename="'.basename($fname).'"' );
  readfile($fname);
  exit(0);
}


/** Empty document. Used for void-return XMLHttpRequest calls. */
function doc_empty()
{
  global $admin_message_text;
  ?>
  <html><body><?php echo $admin_message_text ?></body></html>
  <?php
}


function main()
{
  // Read globals
  global $dirs,$img,$isize,$tsize,$options;
  if(array_key_exists('f',$_GET) && count($dirs->imagefiles))
  {
    for($img=count($dirs->imagefiles)-1; $img>0; $img--)
        if($_GET['f']==$dirs->imagefiles[$img]->info['basename'])
            break;
  }
  else
  {
    $img=intval($_GET['img']);
  }
  $isize=get_set_cookie('isize',0,5,3);
  $tsize=get_set_cookie('tsize');

  if(array_key_exists('do',$_POST))
    process_form();
  elseif(array_key_exists('dl',$_GET))
    download();

  $doc=$_GET['d'];
  if(0==strlen($options->passwd) || 'a'==$doc)
    doc_admin();
  elseif('e'==$doc)
    doc_empty();
  elseif('l'==$doc)
    doc_latest();
  elseif(0==count($dirs->imagefiles)) // No images
    doc_dir($dirs->directories);
  elseif('x'==$doc)
    doc_xml($dirs->imagefiles);
  elseif('s'==$doc)
    doc_single($dirs->imagefiles);
  elseif('c'==$doc)
    doc_carousel($dirs->imagefiles);
  elseif('t'==$doc || 0==count($dirs->directories))
    doc_thumb($dirs->imagefiles);
  else
    doc_dir($dirs->directories);
}
main();

?>
